/*
  NES Controller Interfacing Using SPI

  Get the current button state from any NES controller and print it over serial.
  The example was crafted with the Joy Boy project [0] in mind, with the end
  goal of connecting a Beeshu Zinger NES Joystick to a Game Boy.

  Hook up your Zinger (or other NES compatible controller) using the
  table below as guidance. A pinout diagram of the Zinger [1] is also available.

  |   Arduino Uno  |      13     |     12    |      8      |  5V | GND |
  |:--------------:|:-----------:|:---------:|:-----------:|:---:|:---:|
  | NES Controller | CLK (clock) | D0 (data) | OUT (latch) | +5V | GND |

  [0]: https://hackaday.io/project/167042-joy-boy
  [1]: https://hackaday.io/project/167042-joy-boy/log/166907-a-colorful-plug-teardown
*/

#include <SPI.h>

// By keeping track of the previous controller state, we only need to print
// changing state. This way, the serial monitor is less chatty.
String previousState = "0b11111111";

// The NESController allows you to interface any NES controller, using the
// Arduino SPI library.
class NESController {
  public:
    void begin(int pin);
    String getStateAsBinaryString();
    byte latch();
  private:
    int latchPin;
} myController;

// Initialize serial communication and controller.
void setup() {
  Serial.begin(9600);

  myController.begin(8);
}

// Update controller state at roughly 60 Hz. Print the current state to serial
// when it changes.
void loop() {
  String state = myController.getStateAsBinaryString();

  if (!state.equals(previousState)) {
    Serial.println(state);
    previousState = state;
  }

  delay(16);
}

// Initializes the SPI bus and latch pin, passed in as a parameter.
void NESController::begin(int latchPin) {
  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_DIV128);

  pinMode(latchPin, OUTPUT);
  digitalWrite(latchPin, LOW);

  this->latchPin = latchPin;
}

// Latches the controller and performs a SPI transfer. Returns button state as a
// single byte.
//
// Each bit in the returned byte represents the state of one controller button.
// A bit set to 1 means button is pressed. A 0 bit means the button is released.
//
// | Bit    | 7 | 6 | 5      | 4     | 3  | 2    | 1    | 0     |
// |--------|---|---|--------|-------|----|------|------|-------|
// | Button | A | B | Select | Start | Up | Down | Left | Right |
byte NESController::latch(void) {
  digitalWrite(latchPin, HIGH);
  delayMicroseconds(12);
  digitalWrite(latchPin, LOW);

  return ~SPI.transfer(0x00);
}

// Calls NESController::latch and returns the result as a binary String using
// the format 0b00000000.
String NESController::getStateAsBinaryString(void) {
  String binary = "0b";
  byte state = this->latch();

  for (int i = 7; i >= 0; i--) {
    binary.concat(bitRead(state, i));
  }

  return binary;
}
